# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import encode_path_param
from ...core.pagination import AsyncPager, SyncPager
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.payment_required_error import PaymentRequiredError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.acul_configs import AculConfigs
from ...types.acul_context_configuration import AculContextConfiguration
from ...types.acul_filters import AculFilters
from ...types.acul_head_tag import AculHeadTag
from ...types.acul_rendering_mode_enum import AculRenderingModeEnum
from ...types.bulk_update_acul_response_content import BulkUpdateAculResponseContent
from ...types.get_acul_response_content import GetAculResponseContent
from ...types.list_aculs_offset_paginated_response_content import ListAculsOffsetPaginatedResponseContent
from ...types.list_aculs_response_content_item import ListAculsResponseContentItem
from ...types.prompt_group_name_enum import PromptGroupNameEnum
from ...types.screen_group_name_enum import ScreenGroupNameEnum
from ...types.update_acul_response_content import UpdateAculResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRenderingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        prompt: typing.Optional[str] = None,
        screen: typing.Optional[str] = None,
        rendering_mode: typing.Optional[AculRenderingModeEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ListAculsResponseContentItem, ListAculsOffsetPaginatedResponseContent]:
        """
        Get render setting configurations for all screens.

        Parameters
        ----------
        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (default: true) or excluded (false).

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Maximum value is 100, default value is 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total configuration count (true) or as a direct array of results (false, default).

        prompt : typing.Optional[str]
            Name of the prompt to filter by

        screen : typing.Optional[str]
            Name of the screen to filter by

        rendering_mode : typing.Optional[AculRenderingModeEnum]
            Rendering mode to filter by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ListAculsResponseContentItem, ListAculsOffsetPaginatedResponseContent]
            ACUL settings successfully retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "prompts/rendering",
            method="GET",
            params={
                "fields": fields,
                "include_fields": include_fields,
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
                "prompt": prompt,
                "screen": screen,
                "rendering_mode": rendering_mode,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListAculsOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListAculsOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.configs
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    fields=fields,
                    include_fields=include_fields,
                    page=page + 1,
                    per_page=per_page,
                    include_totals=include_totals,
                    prompt=prompt,
                    screen=screen,
                    rendering_mode=rendering_mode,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def bulk_update(
        self, *, configs: AculConfigs, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BulkUpdateAculResponseContent]:
        """
        Learn more about [configuring render settings](https://auth0.com/docs/customize/login-pages/advanced-customizations/getting-started/configure-acul-screens) for advanced customization.

        Parameters
        ----------
        configs : AculConfigs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BulkUpdateAculResponseContent]
            ACUL settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "prompts/rendering",
            method="PATCH",
            json={
                "configs": convert_and_respect_annotation_metadata(
                    object_=configs, annotation=AculConfigs, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkUpdateAculResponseContent,
                    parse_obj_as(
                        type_=BulkUpdateAculResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        prompt: PromptGroupNameEnum,
        screen: ScreenGroupNameEnum,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetAculResponseContent]:
        """
        Get render settings for a screen.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt

        screen : ScreenGroupNameEnum
            Name of the screen

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetAculResponseContent]
            ACUL settings successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"prompts/{encode_path_param(prompt)}/screen/{encode_path_param(screen)}/rendering",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAculResponseContent,
                    parse_obj_as(
                        type_=GetAculResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        prompt: PromptGroupNameEnum,
        screen: ScreenGroupNameEnum,
        *,
        rendering_mode: typing.Optional[AculRenderingModeEnum] = OMIT,
        context_configuration: typing.Optional[AculContextConfiguration] = OMIT,
        default_head_tags_disabled: typing.Optional[bool] = OMIT,
        use_page_template: typing.Optional[bool] = OMIT,
        head_tags: typing.Optional[typing.Sequence[AculHeadTag]] = OMIT,
        filters: typing.Optional[AculFilters] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateAculResponseContent]:
        """
        Learn more about [configuring render settings](https://auth0.com/docs/customize/login-pages/advanced-customizations/getting-started/configure-acul-screens) for advanced customization.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt

        screen : ScreenGroupNameEnum
            Name of the screen

        rendering_mode : typing.Optional[AculRenderingModeEnum]
            Rendering mode

        context_configuration : typing.Optional[AculContextConfiguration]

        default_head_tags_disabled : typing.Optional[bool]
            Override Universal Login default head tags

        use_page_template : typing.Optional[bool]
            Use page template with ACUL

        head_tags : typing.Optional[typing.Sequence[AculHeadTag]]
            An array of head tags

        filters : typing.Optional[AculFilters]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateAculResponseContent]
            ACUL settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"prompts/{encode_path_param(prompt)}/screen/{encode_path_param(screen)}/rendering",
            method="PATCH",
            json={
                "rendering_mode": rendering_mode,
                "context_configuration": convert_and_respect_annotation_metadata(
                    object_=context_configuration,
                    annotation=typing.Optional[AculContextConfiguration],
                    direction="write",
                ),
                "default_head_tags_disabled": default_head_tags_disabled,
                "use_page_template": use_page_template,
                "head_tags": convert_and_respect_annotation_metadata(
                    object_=head_tags, annotation=typing.Optional[typing.Sequence[AculHeadTag]], direction="write"
                ),
                "filters": convert_and_respect_annotation_metadata(
                    object_=filters, annotation=typing.Optional[AculFilters], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateAculResponseContent,
                    parse_obj_as(
                        type_=UpdateAculResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRenderingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        prompt: typing.Optional[str] = None,
        screen: typing.Optional[str] = None,
        rendering_mode: typing.Optional[AculRenderingModeEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ListAculsResponseContentItem, ListAculsOffsetPaginatedResponseContent]:
        """
        Get render setting configurations for all screens.

        Parameters
        ----------
        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (default: true) or excluded (false).

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Maximum value is 100, default value is 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total configuration count (true) or as a direct array of results (false, default).

        prompt : typing.Optional[str]
            Name of the prompt to filter by

        screen : typing.Optional[str]
            Name of the screen to filter by

        rendering_mode : typing.Optional[AculRenderingModeEnum]
            Rendering mode to filter by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ListAculsResponseContentItem, ListAculsOffsetPaginatedResponseContent]
            ACUL settings successfully retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "prompts/rendering",
            method="GET",
            params={
                "fields": fields,
                "include_fields": include_fields,
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
                "prompt": prompt,
                "screen": screen,
                "rendering_mode": rendering_mode,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListAculsOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListAculsOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.configs
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        fields=fields,
                        include_fields=include_fields,
                        page=page + 1,
                        per_page=per_page,
                        include_totals=include_totals,
                        prompt=prompt,
                        screen=screen,
                        rendering_mode=rendering_mode,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def bulk_update(
        self, *, configs: AculConfigs, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BulkUpdateAculResponseContent]:
        """
        Learn more about [configuring render settings](https://auth0.com/docs/customize/login-pages/advanced-customizations/getting-started/configure-acul-screens) for advanced customization.

        Parameters
        ----------
        configs : AculConfigs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BulkUpdateAculResponseContent]
            ACUL settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "prompts/rendering",
            method="PATCH",
            json={
                "configs": convert_and_respect_annotation_metadata(
                    object_=configs, annotation=AculConfigs, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkUpdateAculResponseContent,
                    parse_obj_as(
                        type_=BulkUpdateAculResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        prompt: PromptGroupNameEnum,
        screen: ScreenGroupNameEnum,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetAculResponseContent]:
        """
        Get render settings for a screen.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt

        screen : ScreenGroupNameEnum
            Name of the screen

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetAculResponseContent]
            ACUL settings successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"prompts/{encode_path_param(prompt)}/screen/{encode_path_param(screen)}/rendering",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAculResponseContent,
                    parse_obj_as(
                        type_=GetAculResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        prompt: PromptGroupNameEnum,
        screen: ScreenGroupNameEnum,
        *,
        rendering_mode: typing.Optional[AculRenderingModeEnum] = OMIT,
        context_configuration: typing.Optional[AculContextConfiguration] = OMIT,
        default_head_tags_disabled: typing.Optional[bool] = OMIT,
        use_page_template: typing.Optional[bool] = OMIT,
        head_tags: typing.Optional[typing.Sequence[AculHeadTag]] = OMIT,
        filters: typing.Optional[AculFilters] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateAculResponseContent]:
        """
        Learn more about [configuring render settings](https://auth0.com/docs/customize/login-pages/advanced-customizations/getting-started/configure-acul-screens) for advanced customization.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt

        screen : ScreenGroupNameEnum
            Name of the screen

        rendering_mode : typing.Optional[AculRenderingModeEnum]
            Rendering mode

        context_configuration : typing.Optional[AculContextConfiguration]

        default_head_tags_disabled : typing.Optional[bool]
            Override Universal Login default head tags

        use_page_template : typing.Optional[bool]
            Use page template with ACUL

        head_tags : typing.Optional[typing.Sequence[AculHeadTag]]
            An array of head tags

        filters : typing.Optional[AculFilters]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateAculResponseContent]
            ACUL settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"prompts/{encode_path_param(prompt)}/screen/{encode_path_param(screen)}/rendering",
            method="PATCH",
            json={
                "rendering_mode": rendering_mode,
                "context_configuration": convert_and_respect_annotation_metadata(
                    object_=context_configuration,
                    annotation=typing.Optional[AculContextConfiguration],
                    direction="write",
                ),
                "default_head_tags_disabled": default_head_tags_disabled,
                "use_page_template": use_page_template,
                "head_tags": convert_and_respect_annotation_metadata(
                    object_=head_tags, annotation=typing.Optional[typing.Sequence[AculHeadTag]], direction="write"
                ),
                "filters": convert_and_respect_annotation_metadata(
                    object_=filters, annotation=typing.Optional[AculFilters], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateAculResponseContent,
                    parse_obj_as(
                        type_=UpdateAculResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 402:
                raise PaymentRequiredError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_connection_created_object_3_options_assertion_decryption_settings_algorithm_profile_enum import (
    EventStreamCloudEventConnectionCreatedObject3OptionsAssertionDecryptionSettingsAlgorithmProfileEnum,
)


class EventStreamCloudEventConnectionCreatedObject3OptionsAssertionDecryptionSettings(UniversalBaseModel):
    """
    Settings for SAML assertion decryption.
    """

    algorithm_exceptions: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of insecure algorithms to allow for SAML assertion decryption.
    """

    algorithm_profile: (
        EventStreamCloudEventConnectionCreatedObject3OptionsAssertionDecryptionSettingsAlgorithmProfileEnum
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionCreatedObject1OptionsUpstreamParams = typing.Dict[str, typing.Any]
"""
Additional parameters to include in authorization requests sent to the identity provider. Useful for passing custom claims, selecting specific identity sources, or configuring provider-specific behavior. See https://auth0.com/docs/authenticate/identity-providers/pass-parameters-to-idps
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .refresh_token_date import RefreshTokenDate
from .refresh_token_device import RefreshTokenDevice
from .refresh_token_metadata import RefreshTokenMetadata
from .refresh_token_resource_server import RefreshTokenResourceServer
from .refresh_token_session_id import RefreshTokenSessionId


class GetRefreshTokenResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the refresh token
    """

    user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the user which can be used when interacting with other APIs.
    """

    created_at: typing.Optional[RefreshTokenDate] = None
    idle_expires_at: typing.Optional[RefreshTokenDate] = None
    expires_at: typing.Optional[RefreshTokenDate] = None
    device: typing.Optional[RefreshTokenDevice] = None
    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the client application granted with this refresh token
    """

    session_id: typing.Optional[RefreshTokenSessionId] = None
    rotating: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the token is a rotating refresh token
    """

    resource_servers: typing.Optional[typing.List[RefreshTokenResourceServer]] = pydantic.Field(default=None)
    """
    A list of the resource server IDs associated to this refresh-token and their granted scopes
    """

    refresh_token_metadata: typing.Optional[RefreshTokenMetadata] = None
    last_exchanged_at: typing.Optional[RefreshTokenDate] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ConnectionScriptsOAuth2(UniversalBaseModel):
    """
    Custom scripts to transform user profile data or modify OAuth2 flow behavior
    """

    fetch_user_profile: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="fetchUserProfile"),
        pydantic.Field(
            alias="fetchUserProfile",
            description="Custom JavaScript function to retrieve and transform user profile data from the identity provider. Called with the access token and token exchange response. Must return a user profile object. Executed in a sandboxed environment. If not provided, an empty profile object is used.",
        ),
    ] = None
    """
    Custom JavaScript function to retrieve and transform user profile data from the identity provider. Called with the access token and token exchange response. Must return a user profile object. Executed in a sandboxed environment. If not provided, an empty profile object is used.
    """

    get_logout_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="getLogoutUrl"),
        pydantic.Field(
            alias="getLogoutUrl",
            description="Custom JavaScript function to dynamically construct the logout URL for the identity provider. Called with the request query parameters and must invoke a callback with the logout URL. Only used if 'logoutUrl' is not configured. Executed in a sandboxed environment.",
        ),
    ] = None
    """
    Custom JavaScript function to dynamically construct the logout URL for the identity provider. Called with the request query parameters and must invoke a callback with the logout URL. Only used if 'logoutUrl' is not configured. Executed in a sandboxed environment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.list_user_permissions_offset_paginated_response_content import (
    ListUserPermissionsOffsetPaginatedResponseContent,
)
from ...types.permission_request_payload import PermissionRequestPayload
from ...types.user_permission_schema import UserPermissionSchema
from .raw_client import AsyncRawPermissionsClient, RawPermissionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PermissionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPermissionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPermissionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPermissionsClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        per_page: typing.Optional[int] = 50,
        page: typing.Optional[int] = 0,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[UserPermissionSchema, ListUserPermissionsOffsetPaginatedResponseContent]:
        """
        Retrieve all permissions associated with the user.

        Parameters
        ----------
        id : str
            ID of the user to retrieve the permissions for.

        per_page : typing.Optional[int]
            Number of results per page.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[UserPermissionSchema, ListUserPermissionsOffsetPaginatedResponseContent]
            Permissions successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.users.permissions.list(
            id="id",
            per_page=1,
            page=1,
            include_totals=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            id, per_page=per_page, page=page, include_totals=include_totals, request_options=request_options
        )

    def create(
        self,
        id: str,
        *,
        permissions: typing.Sequence[PermissionRequestPayload],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Assign permissions to a user.

        Parameters
        ----------
        id : str
            ID of the user to assign permissions to.

        permissions : typing.Sequence[PermissionRequestPayload]
            List of permissions to add to this user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0, PermissionRequestPayload

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.permissions.create(
            id="id",
            permissions=[
                PermissionRequestPayload(
                    resource_server_identifier="resource_server_identifier",
                    permission_name="permission_name",
                )
            ],
        )
        """
        _response = self._raw_client.create(id, permissions=permissions, request_options=request_options)
        return _response.data

    def delete(
        self,
        id: str,
        *,
        permissions: typing.Sequence[PermissionRequestPayload],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Remove permissions from a user.

        Parameters
        ----------
        id : str
            ID of the user to remove permissions from.

        permissions : typing.Sequence[PermissionRequestPayload]
            List of permissions to remove from this user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0, PermissionRequestPayload

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.permissions.delete(
            id="id",
            permissions=[
                PermissionRequestPayload(
                    resource_server_identifier="resource_server_identifier",
                    permission_name="permission_name",
                )
            ],
        )
        """
        _response = self._raw_client.delete(id, permissions=permissions, request_options=request_options)
        return _response.data


class AsyncPermissionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPermissionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPermissionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPermissionsClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        per_page: typing.Optional[int] = 50,
        page: typing.Optional[int] = 0,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[UserPermissionSchema, ListUserPermissionsOffsetPaginatedResponseContent]:
        """
        Retrieve all permissions associated with the user.

        Parameters
        ----------
        id : str
            ID of the user to retrieve the permissions for.

        per_page : typing.Optional[int]
            Number of results per page.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[UserPermissionSchema, ListUserPermissionsOffsetPaginatedResponseContent]
            Permissions successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.users.permissions.list(
                id="id",
                per_page=1,
                page=1,
                include_totals=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            id, per_page=per_page, page=page, include_totals=include_totals, request_options=request_options
        )

    async def create(
        self,
        id: str,
        *,
        permissions: typing.Sequence[PermissionRequestPayload],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Assign permissions to a user.

        Parameters
        ----------
        id : str
            ID of the user to assign permissions to.

        permissions : typing.Sequence[PermissionRequestPayload]
            List of permissions to add to this user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, PermissionRequestPayload

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.permissions.create(
                id="id",
                permissions=[
                    PermissionRequestPayload(
                        resource_server_identifier="resource_server_identifier",
                        permission_name="permission_name",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(id, permissions=permissions, request_options=request_options)
        return _response.data

    async def delete(
        self,
        id: str,
        *,
        permissions: typing.Sequence[PermissionRequestPayload],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Remove permissions from a user.

        Parameters
        ----------
        id : str
            ID of the user to remove permissions from.

        permissions : typing.Sequence[PermissionRequestPayload]
            List of permissions to remove from this user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, PermissionRequestPayload

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.permissions.delete(
                id="id",
                permissions=[
                    PermissionRequestPayload(
                        resource_server_identifier="resource_server_identifier",
                        permission_name="permission_name",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, permissions=permissions, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionDeletedObject7OptionsIdentityApiEnum = typing.Union[
    typing.Literal["microsoft-identity-platform-v2.0", "azure-active-directory-v1.0"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .acul_context_configuration_item import AculContextConfigurationItem

AculContextConfiguration = typing.Optional[typing.List[AculContextConfigurationItem]]
"""
Context values to make available
"""



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventUserUpdatedObjectAppMetadata = typing.Dict[str, typing.Any]
"""
User metadata to which this user has read-only access.
"""



from __future__ import annotations

from typing import Any

from .base import AuthenticationBase


class Delegated(AuthenticationBase):
    """Delegated authentication endpoints.

    Args:
        domain (str): Your auth0 domain (e.g: username.auth0.com)
    """

    def get_token(
        self,
        target: str,
        api_type: str,
        grant_type: str,
        id_token: str | None = None,
        refresh_token: str | None = None,
        scope: str = "openid",
    ) -> Any:
        """Obtain a delegation token."""

        if id_token and refresh_token:
            raise ValueError("Only one of id_token or refresh_token can be None")

        data = {
            "client_id": self.client_id,
            "grant_type": grant_type,
            "target": target,
            "scope": scope,
            "api_type": api_type,
        }

        if id_token:
            data.update({"id_token": id_token})
        elif refresh_token:
            data.update({"refresh_token": refresh_token})
        else:
            raise ValueError("Either id_token or refresh_token must have a value")

        return self.post(f"{self.protocol}://{self.domain}/delegation", data=data)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class EventStreamCloudEventContextRequestGeo(UniversalBaseModel):
    """
    Geographic information about the request origin.
    """

    continent_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Continent code.
    """

    country_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Country code.
    """

    country_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Country name.
    """

    latitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Latitude coordinate.
    """

    longitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Longitude coordinate.
    """

    subdivision_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Subdivision (state/province) code.
    """

    subdivision_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Subdivision (state/province) name.
    """

    city_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    City name.
    """

    time_zone: typing.Optional[str] = pydantic.Field(default=None)
    """
    Time zone.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import encode_path_param
from ...core.pagination import AsyncPager, SyncPager
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.connection_for_list import ConnectionForList
from ...types.connection_strategy_enum import ConnectionStrategyEnum
from ...types.list_client_connections_response_content import ListClientConnectionsResponseContent
from pydantic import ValidationError


class RawConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        id: str,
        *,
        strategy: typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ConnectionForList, ListClientConnectionsResponseContent]:
        """
        Retrieve all connections that are enabled for the specified [Application](https://www.auth0.com/docs/get-started/applications), using checkpoint pagination. A list of fields to include or exclude for each connection may also be specified.

        - This endpoint requires the `read:connections` scope and any one of `read:clients` or `read:client_summary`.
        - **Note**: The first time you call this endpoint, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no further results are remaining.

        Parameters
        ----------
        id : str
            ID of the client for which to retrieve enabled connections.

        strategy : typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]]
            Provide strategies to only retrieve connections with such strategies

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ConnectionForList, ListClientConnectionsResponseContent]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(id)}/connections",
            method="GET",
            params={
                "strategy": strategy,
                "from": from_,
                "take": take,
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListClientConnectionsResponseContent,
                    parse_obj_as(
                        type_=ListClientConnectionsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.connections
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.get(
                    id,
                    strategy=strategy,
                    from_=_parsed_next,
                    take=take,
                    fields=fields,
                    include_fields=include_fields,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        id: str,
        *,
        strategy: typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ConnectionForList, ListClientConnectionsResponseContent]:
        """
        Retrieve all connections that are enabled for the specified [Application](https://www.auth0.com/docs/get-started/applications), using checkpoint pagination. A list of fields to include or exclude for each connection may also be specified.

        - This endpoint requires the `read:connections` scope and any one of `read:clients` or `read:client_summary`.
        - **Note**: The first time you call this endpoint, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no further results are remaining.

        Parameters
        ----------
        id : str
            ID of the client for which to retrieve enabled connections.

        strategy : typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]]
            Provide strategies to only retrieve connections with such strategies

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ConnectionForList, ListClientConnectionsResponseContent]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(id)}/connections",
            method="GET",
            params={
                "strategy": strategy,
                "from": from_,
                "take": take,
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListClientConnectionsResponseContent,
                    parse_obj_as(
                        type_=ListClientConnectionsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.connections
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""

                async def _get_next():
                    return await self.get(
                        id,
                        strategy=strategy,
                        from_=_parsed_next,
                        take=take,
                        fields=fields,
                        include_fields=include_fields,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .connection_scope_array import ConnectionScopeArray

ConnectionFreeformScopesWindowsLive = ConnectionScopeArray
"""
Custom OAuth scopes not predefined in the standard scope list.
"""



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from ..core.serialization import FieldMetadata


class FlowActionFlowBooleanConditionParams(UniversalBaseModel):
    then: typing.Optional[typing.List["FlowAction"]] = None
    else_: typing_extensions.Annotated[
        typing.Optional[typing.List["FlowAction"]], FieldMetadata(alias="else"), pydantic.Field(alias="else")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .flow_action import FlowAction  # noqa: E402, I001
from .flow_action_flow import FlowActionFlow  # noqa: E402, I001
from .flow_action_flow_boolean_condition import FlowActionFlowBooleanCondition  # noqa: E402, I001

update_forward_refs(
    FlowActionFlowBooleanConditionParams,
    FlowAction=FlowAction,
    FlowActionFlow=FlowActionFlow,
    FlowActionFlowBooleanCondition=FlowActionFlowBooleanCondition,
)



# This file was auto-generated by Fern from our API Definition.

from .connection_https_url_with_http_fallback_2048 import ConnectionHttpsUrlWithHttpFallback2048

ConnectionMetadataUrlSaml = ConnectionHttpsUrlWithHttpFallback2048
"""
HTTPS URL to the identity provider's SAML metadata document. When provided, Auth0 automatically fetches and parses the metadata to extract signInEndpoint, signOutEndpoint, signingCert, signSAMLRequest, and protocolBinding. Use metadataUrl OR metadataXml, not both.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .signing_keys_date import SigningKeysDate


class GetSigningKeysResponseContent(UniversalBaseModel):
    kid: str = pydantic.Field()
    """
    The key id of the signing key
    """

    cert: str = pydantic.Field()
    """
    The public certificate of the signing key
    """

    pkcs_7: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="pkcs7"),
        pydantic.Field(alias="pkcs7", description="The public certificate of the signing key in pkcs7 format"),
    ] = None
    """
    The public certificate of the signing key in pkcs7 format
    """

    current: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the key is the the current key
    """

    next: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the key is the the next key
    """

    previous: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the key is the the previous key
    """

    current_since: typing.Optional[SigningKeysDate] = None
    current_until: typing.Optional[SigningKeysDate] = None
    fingerprint: str = pydantic.Field()
    """
    The cert fingerprint
    """

    thumbprint: str = pydantic.Field()
    """
    The cert thumbprint
    """

    revoked: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the key is revoked
    """

    revoked_at: typing.Optional[SigningKeysDate] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_client_id_windows_live import ConnectionClientIdWindowsLive
from .connection_client_secret_windows_live import ConnectionClientSecretWindowsLive
from .connection_freeform_scopes_windows_live import ConnectionFreeformScopesWindowsLive
from .connection_options_common import ConnectionOptionsCommon
from .connection_scope_array_windows_live import ConnectionScopeArrayWindowsLive
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_strategy_version_enum_windows_live import ConnectionStrategyVersionEnumWindowsLive
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsWindowsLive(ConnectionOptionsCommon):
    """
    Options for the 'windowslive' connection
    """

    client_id: typing.Optional[ConnectionClientIdWindowsLive] = None
    client_secret: typing.Optional[ConnectionClientSecretWindowsLive] = None
    freeform_scopes: typing.Optional[ConnectionFreeformScopesWindowsLive] = None
    scope: typing.Optional[ConnectionScopeArrayWindowsLive] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    strategy_version: typing.Optional[ConnectionStrategyVersionEnumWindowsLive] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    applications: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests access to user's applications.
    """

    applications_create: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to create applications.
    """

    basic: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to user's basic profile information and contacts list.
    """

    birthday: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to user's birth day, month, and year.
    """

    calendars: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to user's calendars and events.
    """

    calendars_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read and write access to user's calendars and events.
    """

    contacts_birthday: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to contacts' birth day and birth month.
    """

    contacts_calendars: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to user's calendars and shared calendars/events from others.
    """

    contacts_create: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to create new contacts in user's address book.
    """

    contacts_photos: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to user's and shared albums, photos, videos, and audio.
    """

    contacts_skydrive: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to OneDrive files shared by other users.
    """

    directory_accessasuser_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to have the same access to information in the directory as the signed-in user.
    """

    directory_read_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read data in your organization's directory, such as users, groups, and apps.
    """

    directory_readwrite_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read and write data in your organization's directory, such as users and groups.
    """

    emails: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to personal, preferred, and business email addresses.
    """

    events_create: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to create events on user's default calendar.
    """

    graph_calendars: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read the user's calendars.
    """

    graph_calendars_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read and write the user's calendars.
    """

    graph_contacts: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read the user's contacts.
    """

    graph_contacts_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read and write the user's contacts.
    """

    graph_device: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read the user's device information.
    """

    graph_device_command: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to send commands to the user's devices.
    """

    graph_emails: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read the user's emails.
    """

    graph_emails_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read and write the user's emails.
    """

    graph_files: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read the user's files.
    """

    graph_files_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read all files the user has access to.
    """

    graph_files_all_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read and write all files the user has access to.
    """

    graph_files_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read and write the user's files.
    """

    graph_notes: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read the user's OneNote notebooks.
    """

    graph_notes_create: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to create new OneNote notebooks.
    """

    graph_notes_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read and write the user's OneNote notebooks.
    """

    graph_tasks: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read the user's tasks.
    """

    graph_tasks_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read and write the user's tasks.
    """

    graph_user: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read the user's profile.
    """

    graph_user_activity: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read the user's activity history.
    """

    graph_user_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to read and write the user's profile.
    """

    group_read_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read all group properties and memberships.
    """

    group_readwrite_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to create groups, read all group properties and memberships, update group properties and memberships, and delete groups.
    """

    mail_readwrite_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to create, read, update, and delete all mail in all mailboxes.
    """

    mail_send: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to send mail as users in the organization.
    """

    messenger: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests access to user's Windows Live Messenger data.
    """

    offline_access: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests a refresh token for offline access.
    """

    phone_numbers: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to personal, business, and mobile phone numbers.
    """

    photos: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to user's photos, videos, audio, and albums.
    """

    postal_addresses: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to personal and business postal addresses.
    """

    rolemanagement_read_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read the role-based access control (RBAC) settings for your company's directory.
    """

    rolemanagement_readwrite_directory: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read and write the role-based access control (RBAC) settings for your company's directory.
    """

    share: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests permission to share content with other users.
    """

    signin: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, provides single sign-in behavior for users already signed into their Microsoft account.
    """

    sites_read_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read documents and list items in all SharePoint site collections.
    """

    sites_readwrite_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to create, read, update, and delete documents and list items in all SharePoint site collections.
    """

    skydrive: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to user's files stored on OneDrive.
    """

    skydrive_update: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read and write access to user's OneDrive files.
    """

    team_readbasic_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read the names and descriptions of all teams.
    """

    team_readwrite_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read and write all teams' information and change team membership.
    """

    user_read_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read the full set of profile properties, reports, and managers of all users.
    """

    user_readbasic_all: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, allows the app to read a basic set of profile properties of all users in the directory.
    """

    work_profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, requests read access to employer and work position information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.get_session_response_content import GetSessionResponseContent
from ..types.session_metadata import SessionMetadata
from ..types.update_session_response_content import UpdateSessionResponseContent
from .raw_client import AsyncRawSessionsClient, RawSessionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SessionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSessionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSessionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSessionsClient
        """
        return self._raw_client

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetSessionResponseContent:
        """
        Retrieve session information.

        Parameters
        ----------
        id : str
            ID of session to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSessionResponseContent
            The session was retrieved

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.sessions.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a session by ID.

        Parameters
        ----------
        id : str
            ID of the session to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.sessions.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        session_metadata: typing.Optional[SessionMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSessionResponseContent:
        """
        Update session information.

        Parameters
        ----------
        id : str
            ID of the session to update.

        session_metadata : typing.Optional[SessionMetadata]
            Metadata associated with the session. Pass null or {} to remove all session_metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSessionResponseContent
            Session successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.sessions.update(
            id="id",
        )
        """
        _response = self._raw_client.update(id, session_metadata=session_metadata, request_options=request_options)
        return _response.data

    def revoke(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Revokes a session by ID and all associated refresh tokens.

        Parameters
        ----------
        id : str
            ID of the session to revoke.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.sessions.revoke(
            id="id",
        )
        """
        _response = self._raw_client.revoke(id, request_options=request_options)
        return _response.data


class AsyncSessionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSessionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSessionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSessionsClient
        """
        return self._raw_client

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSessionResponseContent:
        """
        Retrieve session information.

        Parameters
        ----------
        id : str
            ID of session to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSessionResponseContent
            The session was retrieved

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.sessions.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a session by ID.

        Parameters
        ----------
        id : str
            ID of the session to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.sessions.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        session_metadata: typing.Optional[SessionMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSessionResponseContent:
        """
        Update session information.

        Parameters
        ----------
        id : str
            ID of the session to update.

        session_metadata : typing.Optional[SessionMetadata]
            Metadata associated with the session. Pass null or {} to remove all session_metadata.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSessionResponseContent
            Session successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.sessions.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id, session_metadata=session_metadata, request_options=request_options
        )
        return _response.data

    async def revoke(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Revokes a session by ID and all associated refresh tokens.

        Parameters
        ----------
        id : str
            ID of the session to revoke.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.sessions.revoke(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.revoke(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

EmailSpecificProviderSettingsWithAdditionalProperties = typing.Optional[typing.Dict[str, typing.Any]]
"""
Specific provider setting
"""



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawVerifiableCredentialsClient, RawVerifiableCredentialsClient

if typing.TYPE_CHECKING:
    from .verification.client import AsyncVerificationClient, VerificationClient


class VerifiableCredentialsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVerifiableCredentialsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._verification: typing.Optional[VerificationClient] = None

    @property
    def with_raw_response(self) -> RawVerifiableCredentialsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVerifiableCredentialsClient
        """
        return self._raw_client

    @property
    def verification(self):
        if self._verification is None:
            from .verification.client import VerificationClient  # noqa: E402

            self._verification = VerificationClient(client_wrapper=self._client_wrapper)
        return self._verification


class AsyncVerifiableCredentialsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVerifiableCredentialsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._verification: typing.Optional[AsyncVerificationClient] = None

    @property
    def with_raw_response(self) -> AsyncRawVerifiableCredentialsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVerifiableCredentialsClient
        """
        return self._raw_client

    @property
    def verification(self):
        if self._verification is None:
            from .verification.client import AsyncVerificationClient  # noqa: E402

            self._verification = AsyncVerificationClient(client_wrapper=self._client_wrapper)
        return self._verification



from .conftest import get_client, verify_request_count


def test_userBlocks_list_by_identifier() -> None:
    """Test listByIdentifier endpoint with WireMock"""
    test_id = "user_blocks.list_by_identifier.0"
    client = get_client(test_id)
    client.user_blocks.list_by_identifier(
        identifier="identifier",
        consider_brute_force_enablement=True,
    )
    verify_request_count(
        test_id, "GET", "/user-blocks", {"identifier": "identifier", "consider_brute_force_enablement": "true"}, 1
    )


def test_userBlocks_delete_by_identifier() -> None:
    """Test deleteByIdentifier endpoint with WireMock"""
    test_id = "user_blocks.delete_by_identifier.0"
    client = get_client(test_id)
    client.user_blocks.delete_by_identifier(
        identifier="identifier",
    )
    verify_request_count(test_id, "DELETE", "/user-blocks", {"identifier": "identifier"}, 1)


def test_userBlocks_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "user_blocks.list_.0"
    client = get_client(test_id)
    client.user_blocks.list(
        id="id",
        consider_brute_force_enablement=True,
    )
    verify_request_count(test_id, "GET", "/user-blocks/id", {"consider_brute_force_enablement": "true"}, 1)


def test_userBlocks_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "user_blocks.delete.0"
    client = get_client(test_id)
    client.user_blocks.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/user-blocks/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionAllowedAudiencesGoogleOAuth2 = typing.List[str]
"""
List of allowed audiences in the ID token for Google Native Social Login
"""



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.get_group_response_content import GetGroupResponseContent
from ..types.group import Group
from ..types.list_groups_paginated_response_content import ListGroupsPaginatedResponseContent
from .raw_client import AsyncRawGroupsClient, RawGroupsClient

if typing.TYPE_CHECKING:
    from .members.client import AsyncMembersClient, MembersClient
    from .roles.client import AsyncRolesClient, RolesClient


class GroupsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGroupsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._members: typing.Optional[MembersClient] = None
        self._roles: typing.Optional[RolesClient] = None

    @property
    def with_raw_response(self) -> RawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGroupsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        connection_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        external_id: typing.Optional[str] = None,
        search: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Group, ListGroupsPaginatedResponseContent]:
        """
        List all groups in your tenant.

        Parameters
        ----------
        connection_id : typing.Optional[str]
            Filter groups by connection ID.

        name : typing.Optional[str]
            Filter groups by name.

        external_id : typing.Optional[str]
            Filter groups by external ID.

        search : typing.Optional[str]
            Search for groups by name or external ID.

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Group, ListGroupsPaginatedResponseContent]
            Groups successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.groups.list(
            connection_id="connection_id",
            name="name",
            external_id="external_id",
            search="search",
            fields="fields",
            include_fields=True,
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            connection_id=connection_id,
            name=name,
            external_id=external_id,
            search=search,
            fields=fields,
            include_fields=include_fields,
            from_=from_,
            take=take,
            request_options=request_options,
        )

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetGroupResponseContent:
        """
        Retrieve a group by its ID.

        Parameters
        ----------
        id : str
            Unique identifier for the group (service-generated).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGroupResponseContent
            Group successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.groups.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a group by its ID.

        Parameters
        ----------
        id : str
            Unique identifier for the group (service-generated).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.groups.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    @property
    def members(self):
        if self._members is None:
            from .members.client import MembersClient  # noqa: E402

            self._members = MembersClient(client_wrapper=self._client_wrapper)
        return self._members

    @property
    def roles(self):
        if self._roles is None:
            from .roles.client import RolesClient  # noqa: E402

            self._roles = RolesClient(client_wrapper=self._client_wrapper)
        return self._roles


class AsyncGroupsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGroupsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._members: typing.Optional[AsyncMembersClient] = None
        self._roles: typing.Optional[AsyncRolesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGroupsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        connection_id: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        external_id: typing.Optional[str] = None,
        search: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Group, ListGroupsPaginatedResponseContent]:
        """
        List all groups in your tenant.

        Parameters
        ----------
        connection_id : typing.Optional[str]
            Filter groups by connection ID.

        name : typing.Optional[str]
            Filter groups by name.

        external_id : typing.Optional[str]
            Filter groups by external ID.

        search : typing.Optional[str]
            Search for groups by name or external ID.

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Group, ListGroupsPaginatedResponseContent]
            Groups successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.groups.list(
                connection_id="connection_id",
                name="name",
                external_id="external_id",
                search="search",
                fields="fields",
                include_fields=True,
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            connection_id=connection_id,
            name=name,
            external_id=external_id,
            search=search,
            fields=fields,
            include_fields=include_fields,
            from_=from_,
            take=take,
            request_options=request_options,
        )

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetGroupResponseContent:
        """
        Retrieve a group by its ID.

        Parameters
        ----------
        id : str
            Unique identifier for the group (service-generated).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGroupResponseContent
            Group successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.groups.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a group by its ID.

        Parameters
        ----------
        id : str
            Unique identifier for the group (service-generated).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.groups.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    @property
    def members(self):
        if self._members is None:
            from .members.client import AsyncMembersClient  # noqa: E402

            self._members = AsyncMembersClient(client_wrapper=self._client_wrapper)
        return self._members

    @property
    def roles(self):
        if self._roles is None:
            from .roles.client import AsyncRolesClient  # noqa: E402

            self._roles = AsyncRolesClient(client_wrapper=self._client_wrapper)
        return self._roles



# This file was auto-generated by Fern from our API Definition.

NetworkAclActionRedirectEnum = bool
"""
Indicates the rule will redirect requests that either match or not_match specific criteria
"""



# This file was auto-generated by Fern from our API Definition.

import typing

EmailProviderNameEnum = typing.Union[
    typing.Literal[
        "mailgun", "mandrill", "sendgrid", "resend", "ses", "sparkpost", "smtp", "azure_cs", "ms365", "custom"
    ],
    typing.Any,
]



from .conftest import get_client, verify_request_count


def test_tenants_settings_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "tenants.settings.get.0"
    client = get_client(test_id)
    client.tenants.settings.get(
        fields="fields",
        include_fields=True,
    )
    verify_request_count(test_id, "GET", "/tenants/settings", {"fields": "fields", "include_fields": "true"}, 1)


def test_tenants_settings_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "tenants.settings.update.0"
    client = get_client(test_id)
    client.tenants.settings.update()
    verify_request_count(test_id, "PATCH", "/tenants/settings", None, 1)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawRolesClient, RawRolesClient

if typing.TYPE_CHECKING:
    from .members.client import AsyncMembersClient, MembersClient


class RolesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRolesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._members: typing.Optional[MembersClient] = None

    @property
    def with_raw_response(self) -> RawRolesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRolesClient
        """
        return self._raw_client

    @property
    def members(self):
        if self._members is None:
            from .members.client import MembersClient  # noqa: E402

            self._members = MembersClient(client_wrapper=self._client_wrapper)
        return self._members


class AsyncRolesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRolesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._members: typing.Optional[AsyncMembersClient] = None

    @property
    def with_raw_response(self) -> AsyncRawRolesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRolesClient
        """
        return self._raw_client

    @property
    def members(self):
        if self._members is None:
            from .members.client import AsyncMembersClient  # noqa: E402

            self._members = AsyncMembersClient(client_wrapper=self._client_wrapper)
        return self._members



import unittest
from unittest import mock

from auth0.authentication.social import Social


class TestSocial(unittest.TestCase):
    @mock.patch("auth0.authentication.social.Social.post")
    def test_login(self, mock_post):
        s = Social("a.b.c", "cid")
        s.login(access_token="atk", connection="conn")

        args, kwargs = mock_post.call_args

        self.assertEqual("https://a.b.c/oauth/access_token", args[0])
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "access_token": "atk",
                "connection": "conn",
                "scope": "openid",
            },
        )

    @mock.patch("auth0.authentication.social.Social.post")
    def test_login_with_scope(self, mock_post):
        s = Social("a.b.c", "cid")
        s.login(
            access_token="atk",
            connection="conn",
            scope="openid profile",
        )

        args, kwargs = mock_post.call_args

        self.assertEqual("https://a.b.c/oauth/access_token", args[0])
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "access_token": "atk",
                "connection": "conn",
                "scope": "openid profile",
            },
        )



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_mailjet_api_key import CreateFlowsVaultConnectionMailjetApiKey
from .create_flows_vault_connection_mailjet_uninitialized import CreateFlowsVaultConnectionMailjetUninitialized

CreateFlowsVaultConnectionMailjet = typing.Union[
    CreateFlowsVaultConnectionMailjetApiKey, CreateFlowsVaultConnectionMailjetUninitialized
]



# This file was auto-generated by Fern from our API Definition.

import typing

LogStreamPiiLogFieldsEnum = typing.Union[
    typing.Literal["first_name", "last_name", "username", "email", "phone", "address"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.
# Modified by Auth0 to use Auth0 telemetry format with dynamic versioning

import base64
import platform
import sys
import typing
from json import dumps

import httpx
from .http_client import AsyncHttpClient, HttpClient
from .logging import LogConfig, Logger


class BaseClientWrapper:
    def __init__(
        self,
        *,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        base_url: str,
        timeout: typing.Optional[float] = None,
        max_retries: int = 2,
        stream_reconnection_enabled: typing.Optional[bool] = None,
        max_stream_reconnection_attempts: typing.Optional[int] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
    ):
        self._token = token
        self._headers = headers
        self._base_url = base_url
        self._timeout = timeout
        self._max_retries = max_retries
        self._stream_reconnection_enabled = stream_reconnection_enabled
        self._max_stream_reconnection_attempts = max_stream_reconnection_attempts
        self._logging = logging

    def get_headers(self) -> typing.Dict[str, str]:
        py_version = platform.python_version()
        version = sys.modules["auth0"].__version__

        auth0_client = dumps({
            "name": "auth0-python",
            "version": version,
            "env": {"python": py_version}
        }).encode("utf-8")

        headers: typing.Dict[str, str] = {
            "User-Agent": f"Python/{py_version}",
            "Auth0-Client": base64.b64encode(auth0_client).decode(),
            **(self.get_custom_headers() or {}),
        }
        headers["Authorization"] = f"Bearer {self._get_token()}"
        return headers

    def _get_token(self) -> str:
        if isinstance(self._token, str):
            return self._token
        else:
            return self._token()

    def get_custom_headers(self) -> typing.Optional[typing.Dict[str, str]]:
        return self._headers

    def get_base_url(self) -> str:
        return self._base_url

    def get_timeout(self) -> typing.Optional[float]:
        return self._timeout

    def get_max_retries(self) -> int:
        return self._max_retries

    def get_stream_reconnection_enabled(self) -> bool:
        return self._stream_reconnection_enabled if self._stream_reconnection_enabled is not None else True

    def get_max_stream_reconnection_attempts(self) -> typing.Optional[int]:
        return self._max_stream_reconnection_attempts


class SyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        base_url: str,
        timeout: typing.Optional[float] = None,
        max_retries: int = 2,
        stream_reconnection_enabled: typing.Optional[bool] = None,
        max_stream_reconnection_attempts: typing.Optional[int] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
        httpx_client: httpx.Client,
    ):
        super().__init__(
            token=token,
            headers=headers,
            base_url=base_url,
            timeout=timeout,
            max_retries=max_retries,
            stream_reconnection_enabled=stream_reconnection_enabled,
            max_stream_reconnection_attempts=max_stream_reconnection_attempts,
            logging=logging,
        )
        self.httpx_client = HttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
            base_max_retries=self.get_max_retries(),
            logging_config=self._logging,
        )


class AsyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        base_url: str,
        timeout: typing.Optional[float] = None,
        max_retries: int = 2,
        stream_reconnection_enabled: typing.Optional[bool] = None,
        max_stream_reconnection_attempts: typing.Optional[int] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
        async_token: typing.Optional[typing.Callable[[], typing.Awaitable[str]]] = None,
        httpx_client: httpx.AsyncClient,
    ):
        super().__init__(
            token=token,
            headers=headers,
            base_url=base_url,
            timeout=timeout,
            max_retries=max_retries,
            stream_reconnection_enabled=stream_reconnection_enabled,
            max_stream_reconnection_attempts=max_stream_reconnection_attempts,
            logging=logging,
        )
        self._async_token = async_token
        self.httpx_client = AsyncHttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
            base_max_retries=self.get_max_retries(),
            async_base_headers=self.async_get_headers,
            logging_config=self._logging,
        )

    async def async_get_headers(self) -> typing.Dict[str, str]:
        headers = self.get_headers()
        if self._async_token is not None:
            token = await self._async_token()
            headers["Authorization"] = f"Bearer {token}"
        return headers



# This file was auto-generated by Fern from our API Definition.

import typing

from .connection_auth_params_additional_properties_o_auth_2 import ConnectionAuthParamsAdditionalPropertiesOAuth2

ConnectionAuthParamsOAuth2 = typing.Dict[str, ConnectionAuthParamsAdditionalPropertiesOAuth2]
"""
Additional static parameters included in every authorization request to the identity provider. These parameters are merged with runtime parameters before the authorization redirect. Keys and values are passed as-is to the identity provider's authorization endpoint. See https://auth0.com/docs/authenticate/identity-providers/social-identity-providers/oauth2#pass-static-parameters
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionUiLocalesSupported = typing.List[str]
"""
Languages and scripts supported for the user interface, represented as a JSON array of BCP47 [RFC5646] language tag values.
"""



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .refresh_token_date_object import RefreshTokenDateObject

RefreshTokenDate = typing.Union[dt.datetime, RefreshTokenDateObject]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_authorization_endpoint import ConnectionAuthorizationEndpoint
from .connection_client_id_oidc import ConnectionClientIdOidc
from .connection_client_secret_oidc import ConnectionClientSecretOidc
from .connection_connection_settings import ConnectionConnectionSettings
from .connection_domain_aliases import ConnectionDomainAliases
from .connection_dpop_signing_alg_enum import ConnectionDpopSigningAlgEnum
from .connection_icon_url import ConnectionIconUrl
from .connection_id_token_session_expiry_supported import ConnectionIdTokenSessionExpirySupported
from .connection_id_token_signed_response_algs import ConnectionIdTokenSignedResponseAlgs
from .connection_issuer import ConnectionIssuer
from .connection_jwks_uri import ConnectionJwksUri
from .connection_options_oidc_metadata import ConnectionOptionsOidcMetadata
from .connection_scope_oidc import ConnectionScopeOidc
from .connection_send_back_channel_nonce import ConnectionSendBackChannelNonce
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_tenant_domain import ConnectionTenantDomain
from .connection_token_endpoint import ConnectionTokenEndpoint
from .connection_token_endpoint_auth_method_enum import ConnectionTokenEndpointAuthMethodEnum
from .connection_token_endpoint_auth_signing_alg_enum import ConnectionTokenEndpointAuthSigningAlgEnum
from .connection_token_endpoint_jwtca_aud_format_enum_oidc import ConnectionTokenEndpointJwtcaAudFormatEnumOidc
from .connection_upstream_params import ConnectionUpstreamParams
from .connection_userinfo_endpoint import ConnectionUserinfoEndpoint


class ConnectionOptionsCommonOidc(UniversalBaseModel):
    """
    common options for OIDC connections
    """

    authorization_endpoint: typing.Optional[ConnectionAuthorizationEndpoint] = None
    client_id: ConnectionClientIdOidc
    client_secret: typing.Optional[ConnectionClientSecretOidc] = None
    connection_settings: typing.Optional[ConnectionConnectionSettings] = None
    domain_aliases: typing.Optional[ConnectionDomainAliases] = None
    dpop_signing_alg: typing.Optional[ConnectionDpopSigningAlgEnum] = None
    icon_url: typing.Optional[ConnectionIconUrl] = None
    id_token_session_expiry_supported: typing.Optional[ConnectionIdTokenSessionExpirySupported] = None
    id_token_signed_response_algs: typing.Optional[ConnectionIdTokenSignedResponseAlgs] = None
    issuer: typing.Optional[ConnectionIssuer] = None
    jwks_uri: typing.Optional[ConnectionJwksUri] = None
    oidc_metadata: typing.Optional[ConnectionOptionsOidcMetadata] = None
    scope: typing.Optional[ConnectionScopeOidc] = None
    send_back_channel_nonce: typing.Optional[ConnectionSendBackChannelNonce] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    tenant_domain: typing.Optional[ConnectionTenantDomain] = None
    token_endpoint: typing.Optional[ConnectionTokenEndpoint] = None
    token_endpoint_auth_method: typing.Optional[ConnectionTokenEndpointAuthMethodEnum] = None
    token_endpoint_auth_signing_alg: typing.Optional[ConnectionTokenEndpointAuthSigningAlgEnum] = None
    token_endpoint_jwtca_aud_format: typing.Optional[ConnectionTokenEndpointJwtcaAudFormatEnumOidc] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    userinfo_endpoint: typing.Optional[ConnectionUserinfoEndpoint] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .custom_provider_credentials import CustomProviderCredentials
from .twilio_provider_credentials import TwilioProviderCredentials

PhoneProviderCredentials = typing.Union[TwilioProviderCredentials, CustomProviderCredentials]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .suspicious_ip_throttling_allowlist import SuspiciousIpThrottlingAllowlist
from .suspicious_ip_throttling_shields_enum import SuspiciousIpThrottlingShieldsEnum
from .suspicious_ip_throttling_stage import SuspiciousIpThrottlingStage


class UpdateSuspiciousIpThrottlingSettingsResponseContent(UniversalBaseModel):
    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether or not suspicious IP throttling attack protections are active.
    """

    shields: typing.Optional[typing.List[SuspiciousIpThrottlingShieldsEnum]] = pydantic.Field(default=None)
    """
    Action to take when a suspicious IP throttling threshold is violated.
              Possible values: <code>block</code>, <code>admin_notification</code>.
    """

    allowlist: typing.Optional[SuspiciousIpThrottlingAllowlist] = None
    stage: typing.Optional[SuspiciousIpThrottlingStage] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .form_component_category_field_const import FormComponentCategoryFieldConst
from .form_field_tel_config import FormFieldTelConfig
from .form_field_type_tel_const import FormFieldTypeTelConst


class FormFieldTel(UniversalBaseModel):
    id: str
    category: FormComponentCategoryFieldConst
    type: FormFieldTypeTelConst
    config: typing.Optional[FormFieldTelConfig] = None
    label: typing.Optional[str] = None
    hint: typing.Optional[str] = None
    required: typing.Optional[bool] = None
    sensitive: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_airtable_create_record import FlowActionAirtableCreateRecord
from .flow_action_airtable_list_records import FlowActionAirtableListRecords
from .flow_action_airtable_update_record import FlowActionAirtableUpdateRecord

FlowActionAirtable = typing.Union[
    FlowActionAirtableCreateRecord, FlowActionAirtableListRecords, FlowActionAirtableUpdateRecord
]



import unittest
import json
from unittest import mock

from auth0.authentication.pushed_authorization_requests import PushedAuthorizationRequests


class TestRevokeToken(unittest.TestCase):
    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_par(self, mock_post):
        a = PushedAuthorizationRequests("my.domain.com", "cid", client_secret="sh!")
        a.pushed_authorization_request(
            response_type="code", redirect_uri="https://example.com/callback"
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/par")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "sh!",
                "response_type": "code",
                "redirect_uri": "https://example.com/callback",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_par_custom_params(self, mock_post):
        a = PushedAuthorizationRequests("my.domain.com", "cid", client_secret="sh!")
        a.pushed_authorization_request(
            response_type="code", redirect_uri="https://example.com/callback", foo="bar"
        )

        args, kwargs = mock_post.call_args

        self.assertEqual(args[0], "https://my.domain.com/oauth/par")
        self.assertEqual(
            kwargs["data"],
            {
                "client_id": "cid",
                "client_secret": "sh!",
                "response_type": "code",
                "redirect_uri": "https://example.com/callback",
                "foo": "bar",
            },
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_with_authorization_details(self, mock_post):
        a = PushedAuthorizationRequests("my.domain.com", "cid", client_secret="sh!")
        a.pushed_authorization_request(
            response_type="code", 
            redirect_uri="https://example.com/callback", 
            authorization_details=[{"type": "money_transfer", "instructedAmount": {"amount": 2500, "currency": "USD"}}],
        )

        args, kwargs = mock_post.call_args

        expected_data = {
            "client_id": "cid",
            "client_secret": "sh!",
            "response_type": "code",
            "redirect_uri": "https://example.com/callback",
            "authorization_details": [{"type": "money_transfer", "instructedAmount": {"amount": 2500, "currency": "USD"}}],
        }

        actual_data = kwargs["data"]
        
        self.assertEqual(args[0], "https://my.domain.com/oauth/par")
   
        self.assertEqual(
            json.dumps(actual_data, sort_keys=True), 
            json.dumps(expected_data, sort_keys=True)
        )

    @mock.patch("auth0.authentication.rest.RestClient.post")
    def test_jar(self, mock_post):
        a = PushedAuthorizationRequests("my.domain.com", "cid", client_secret="sh!")
        a.pushed_authorization_request(
            response_type="code", 
            redirect_uri="https://example.com/callback", 
            request="my-jwt-request",
        )

        args, kwargs = mock_post.call_args

        expected_data = {
            "client_id": "cid",
            "client_secret": "sh!",
            "response_type": "code",
            "redirect_uri": "https://example.com/callback",
            "request": 'my-jwt-request',
        }

        actual_data = kwargs["data"]
        
        self.assertEqual(args[0], "https://my.domain.com/oauth/par")
   
        self.assertEqual(
            json.dumps(actual_data, sort_keys=True), 
            json.dumps(expected_data, sort_keys=True)
        )


# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class FlowsVaultConnectionSummary(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    Flows Vault Connection identifier.
    """

    app_id: str = pydantic.Field()
    """
    Flows Vault Connection app identifier.
    """

    name: str = pydantic.Field()
    """
    Flows Vault Connection name.
    """

    account_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Flows Vault Connection custom account name.
    """

    ready: bool = pydantic.Field()
    """
    Whether the Flows Vault Connection is configured.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    The ISO 8601 formatted date when this Flows Vault Connection was created.
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    The ISO 8601 formatted date when this Flows Vault Connection was updated.
    """

    refreshed_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The ISO 8601 formatted date when this Flows Vault Connection was refreshed.
    """

    fingerprint: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventUserUpdatedObjectIdentitiesItemPasswordlessProviderEnum = typing.Union[
    typing.Literal["email", "sms"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.event_stream_subscribe_events_event_type_enum import EventStreamSubscribeEventsEventTypeEnum
from ..types.event_stream_subscribe_events_response_content import EventStreamSubscribeEventsResponseContent
from .raw_client import AsyncRawEventsClient, RawEventsClient


class EventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEventsClient
        """
        return self._raw_client

    def subscribe(
        self,
        *,
        from_: typing.Optional[str] = None,
        from_timestamp: typing.Optional[str] = None,
        event_type: typing.Optional[
            typing.Union[
                EventStreamSubscribeEventsEventTypeEnum, typing.Sequence[EventStreamSubscribeEventsEventTypeEnum]
            ]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[EventStreamSubscribeEventsResponseContent]:
        """
        Subscribe to events via Server-Sent Events (SSE)

        Parameters
        ----------
        from_ : typing.Optional[str]
            Opaque token representing position in the stream. If not provided, stream will start from the latest events.

        from_timestamp : typing.Optional[str]
            RFC-3339 timestamp indicating where to start streaming events from. This should only be used on the initial query when a cursor may not be available. Subsequent requests should use the cursor (from) as it will be more accurate.

        event_type : typing.Optional[typing.Union[EventStreamSubscribeEventsEventTypeEnum, typing.Sequence[EventStreamSubscribeEventsEventTypeEnum]]]
            Event type(s) to listen for. Specify multiple times for multiple types (e.g., ?event_type=user.created&event_type=user.updated). If not provided, all event types will be streamed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[EventStreamSubscribeEventsResponseContent]
            Event stream successfully established.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.events.subscribe(
            from_="from",
            from_timestamp="from_timestamp",
            event_type=["connection.created"],
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.subscribe(
            from_=from_, from_timestamp=from_timestamp, event_type=event_type, request_options=request_options
        ) as r:
            yield from r.data


class AsyncEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEventsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEventsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEventsClient
        """
        return self._raw_client

    async def subscribe(
        self,
        *,
        from_: typing.Optional[str] = None,
        from_timestamp: typing.Optional[str] = None,
        event_type: typing.Optional[
            typing.Union[
                EventStreamSubscribeEventsEventTypeEnum, typing.Sequence[EventStreamSubscribeEventsEventTypeEnum]
            ]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[EventStreamSubscribeEventsResponseContent]:
        """
        Subscribe to events via Server-Sent Events (SSE)

        Parameters
        ----------
        from_ : typing.Optional[str]
            Opaque token representing position in the stream. If not provided, stream will start from the latest events.

        from_timestamp : typing.Optional[str]
            RFC-3339 timestamp indicating where to start streaming events from. This should only be used on the initial query when a cursor may not be available. Subsequent requests should use the cursor (from) as it will be more accurate.

        event_type : typing.Optional[typing.Union[EventStreamSubscribeEventsEventTypeEnum, typing.Sequence[EventStreamSubscribeEventsEventTypeEnum]]]
            Event type(s) to listen for. Specify multiple times for multiple types (e.g., ?event_type=user.created&event_type=user.updated). If not provided, all event types will be streamed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[EventStreamSubscribeEventsResponseContent]
            Event stream successfully established.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.events.subscribe(
                from_="from",
                from_timestamp="from_timestamp",
                event_type=["connection.created"],
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.subscribe(
            from_=from_, from_timestamp=from_timestamp, event_type=event_type, request_options=request_options
        ) as r:
            async for _chunk in r.data:
                yield _chunk



from .conftest import get_client, verify_request_count

from auth0.management import OrganizationInvitationInvitee, OrganizationInvitationInviter


def test_organizations_invitations_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "organizations.invitations.list_.0"
    client = get_client(test_id)
    client.organizations.invitations.list(
        id="id",
        page=1,
        per_page=1,
        include_totals=True,
        fields="fields",
        include_fields=True,
        sort="sort",
    )
    verify_request_count(
        test_id,
        "GET",
        "/organizations/id/invitations",
        {
            "page": "1",
            "per_page": "1",
            "include_totals": "true",
            "fields": "fields",
            "include_fields": "true",
            "sort": "sort",
        },
        1,
    )


def test_organizations_invitations_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "organizations.invitations.create.0"
    client = get_client(test_id)
    client.organizations.invitations.create(
        id="id",
        inviter=OrganizationInvitationInviter(
            name="name",
        ),
        invitee=OrganizationInvitationInvitee(
            email="email",
        ),
        client_id="client_id",
    )
    verify_request_count(test_id, "POST", "/organizations/id/invitations", None, 1)


def test_organizations_invitations_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "organizations.invitations.get.0"
    client = get_client(test_id)
    client.organizations.invitations.get(
        id="id",
        invitation_id="invitation_id",
        fields="fields",
        include_fields=True,
    )
    verify_request_count(
        test_id, "GET", "/organizations/id/invitations/invitation_id", {"fields": "fields", "include_fields": "true"}, 1
    )


def test_organizations_invitations_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "organizations.invitations.delete.0"
    client = get_client(test_id)
    client.organizations.invitations.delete(
        id="id",
        invitation_id="invitation_id",
    )
    verify_request_count(test_id, "DELETE", "/organizations/id/invitations/invitation_id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_google_sheets_oauth_code import CreateFlowsVaultConnectionGoogleSheetsOauthCode
from .create_flows_vault_connection_google_sheets_uninitialized import (
    CreateFlowsVaultConnectionGoogleSheetsUninitialized,
)

CreateFlowsVaultConnectionGoogleSheets = typing.Union[
    CreateFlowsVaultConnectionGoogleSheetsOauthCode, CreateFlowsVaultConnectionGoogleSheetsUninitialized
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_passkey_challenge_ui_enum import ConnectionPasskeyChallengeUiEnum


class ConnectionPasskeyOptions(UniversalBaseModel):
    """
    Options for the passkey authentication method
    """

    challenge_ui: typing.Optional[ConnectionPasskeyChallengeUiEnum] = None
    progressive_enrollment_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables or disables progressive enrollment of passkeys for the connection.
    """

    local_enrollment_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables or disables enrollment prompt for local passkey when user authenticates using a cross-device passkey for the connection.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_user_deleted_object_identities_item_custom import (
    EventStreamCloudEventUserDeletedObjectIdentitiesItemCustom,
)
from .event_stream_cloud_event_user_deleted_object_identities_item_database import (
    EventStreamCloudEventUserDeletedObjectIdentitiesItemDatabase,
)
from .event_stream_cloud_event_user_deleted_object_identities_item_enterprise import (
    EventStreamCloudEventUserDeletedObjectIdentitiesItemEnterprise,
)
from .event_stream_cloud_event_user_deleted_object_identities_item_passwordless import (
    EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordless,
)
from .event_stream_cloud_event_user_deleted_object_identities_item_social import (
    EventStreamCloudEventUserDeletedObjectIdentitiesItemSocial,
)

EventStreamCloudEventUserDeletedObjectIdentitiesItem = typing.Union[
    EventStreamCloudEventUserDeletedObjectIdentitiesItemCustom,
    EventStreamCloudEventUserDeletedObjectIdentitiesItemDatabase,
    EventStreamCloudEventUserDeletedObjectIdentitiesItemEnterprise,
    EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordless,
    EventStreamCloudEventUserDeletedObjectIdentitiesItemSocial,
]



# This file was auto-generated by Fern from our API Definition.

import typing

BrandingThemeIdentifiersPhoneDisplayMaskingEnum = typing.Union[
    typing.Literal["hide_country_code", "mask_digits", "show_all"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionUpdatedObject0OptionsDpopSigningAlgEnum = typing.Union[
    typing.Literal["ES256", "ES384", "ES512", "Ed25519"], typing.Any
]



from .conftest import get_client, verify_request_count


def test_userGrants_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "user_grants.list_.0"
    client = get_client(test_id)
    client.user_grants.list(
        per_page=1,
        page=1,
        include_totals=True,
        user_id="user_id",
        client_id="client_id",
        audience="audience",
    )
    verify_request_count(
        test_id,
        "GET",
        "/grants",
        {
            "per_page": "1",
            "page": "1",
            "include_totals": "true",
            "user_id": "user_id",
            "client_id": "client_id",
            "audience": "audience",
        },
        1,
    )


def test_userGrants_delete_by_user_id() -> None:
    """Test deleteByUserId endpoint with WireMock"""
    test_id = "user_grants.delete_by_user_id.0"
    client = get_client(test_id)
    client.user_grants.delete_by_user_id(
        user_id="user_id",
    )
    verify_request_count(test_id, "DELETE", "/grants", {"user_id": "user_id"}, 1)


def test_userGrants_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "user_grants.delete.0"
    client = get_client(test_id)
    client.user_grants.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/grants/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionRequestObjectSigningAlgValuesSupported = typing.List[str]
"""
JSON array containing a list of the JWS signing algorithms (alg values) supported by the OP for Request Objects, which are described in Section 6.1 of OpenID Connect Core 1.0 [OpenID.Core]. These algorithms are used both when the Request Object is passed by value (using the request parameter) and when it is passed by reference (using the request_uri parameter). Servers SHOULD support none and RS256.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ClientMobileiOs(UniversalBaseModel):
    """
    iOS native app configuration.
    """

    team_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Identifier assigned to the Apple account that signs and uploads the app to the store.
    """

    app_bundle_identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Assigned by developer to the app as its unique identifier inside the store. Usually this is a reverse domain plus the app name, e.g. `com.you.MyApp`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count

from auth0.management import AculConfigsItem


def test_prompts_rendering_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "prompts.rendering.list_.0"
    client = get_client(test_id)
    client.prompts.rendering.list(
        fields="fields",
        include_fields=True,
        page=1,
        per_page=1,
        include_totals=True,
        prompt="prompt",
        screen="screen",
        rendering_mode="advanced",
    )
    verify_request_count(
        test_id,
        "GET",
        "/prompts/rendering",
        {
            "fields": "fields",
            "include_fields": "true",
            "page": "1",
            "per_page": "1",
            "include_totals": "true",
            "prompt": "prompt",
            "screen": "screen",
            "rendering_mode": "advanced",
        },
        1,
    )


def test_prompts_rendering_bulk_update() -> None:
    """Test bulkUpdate endpoint with WireMock"""
    test_id = "prompts.rendering.bulk_update.0"
    client = get_client(test_id)
    client.prompts.rendering.bulk_update(
        configs=[
            AculConfigsItem(
                prompt="login",
                screen="login",
            )
        ],
    )
    verify_request_count(test_id, "PATCH", "/prompts/rendering", None, 1)


def test_prompts_rendering_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "prompts.rendering.get.0"
    client = get_client(test_id)
    client.prompts.rendering.get(
        prompt="login",
        screen="login",
    )
    verify_request_count(test_id, "GET", "/prompts/login/screen/login/rendering", None, 1)


def test_prompts_rendering_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "prompts.rendering.update.0"
    client = get_client(test_id)
    client.prompts.rendering.update(
        prompt="login",
        screen="login",
    )
    verify_request_count(test_id, "PATCH", "/prompts/login/screen/login/rendering", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_connection_created_object_0 import EventStreamCloudEventConnectionCreatedObject0
from .event_stream_cloud_event_connection_created_object_1 import EventStreamCloudEventConnectionCreatedObject1
from .event_stream_cloud_event_connection_created_object_2 import EventStreamCloudEventConnectionCreatedObject2
from .event_stream_cloud_event_connection_created_object_3 import EventStreamCloudEventConnectionCreatedObject3
from .event_stream_cloud_event_connection_created_object_4 import EventStreamCloudEventConnectionCreatedObject4
from .event_stream_cloud_event_connection_created_object_5 import EventStreamCloudEventConnectionCreatedObject5
from .event_stream_cloud_event_connection_created_object_6 import EventStreamCloudEventConnectionCreatedObject6
from .event_stream_cloud_event_connection_created_object_7 import EventStreamCloudEventConnectionCreatedObject7

EventStreamCloudEventConnectionCreatedObject = typing.Union[
    EventStreamCloudEventConnectionCreatedObject0,
    EventStreamCloudEventConnectionCreatedObject1,
    EventStreamCloudEventConnectionCreatedObject2,
    EventStreamCloudEventConnectionCreatedObject3,
    EventStreamCloudEventConnectionCreatedObject4,
    EventStreamCloudEventConnectionCreatedObject5,
    EventStreamCloudEventConnectionCreatedObject6,
    EventStreamCloudEventConnectionCreatedObject7,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .credential_id import CredentialId

ClientAuthenticationMethodPrivateKeyJwtCredentials = typing.List[CredentialId]
"""
A list of unique and previously created credential IDs enabled on the client for Private Key JWT authentication.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ActionSecretRequest(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the particular secret, e.g. API_KEY.
    """

    value: typing.Optional[str] = pydantic.Field(default=None)
    """
    The value of the particular secret, e.g. secret123. A secret's value can only be set upon creation. A secret's value will never be returned by the API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .resource_server_subject_type_authorization_client import ResourceServerSubjectTypeAuthorizationClient
from .resource_server_subject_type_authorization_user import ResourceServerSubjectTypeAuthorizationUser


class ResourceServerSubjectTypeAuthorization(UniversalBaseModel):
    """
    Defines application access permission for a resource server
    """

    user: typing.Optional[ResourceServerSubjectTypeAuthorizationUser] = None
    client: typing.Optional[ResourceServerSubjectTypeAuthorizationClient] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .connection_scope_array import ConnectionScopeArray

ConnectionFreeformScopesAmazon = ConnectionScopeArray
"""
Additional OAuth scopes to request from Amazon beyond the standard profile and postal_code scopes. These scopes must be valid Amazon Login scopes.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .form_block_divider import FormBlockDivider
from .form_block_html import FormBlockHtml
from .form_block_image import FormBlockImage
from .form_block_jump_button import FormBlockJumpButton
from .form_block_next_button import FormBlockNextButton
from .form_block_previous_button import FormBlockPreviousButton
from .form_block_resend_button import FormBlockResendButton
from .form_block_rich_text import FormBlockRichText

FormBlock = typing.Union[
    FormBlockDivider,
    FormBlockHtml,
    FormBlockImage,
    FormBlockJumpButton,
    FormBlockResendButton,
    FormBlockNextButton,
    FormBlockPreviousButton,
    FormBlockRichText,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .connection_identifier_precedence_enum import ConnectionIdentifierPrecedenceEnum

ConnectionIdentifierPrecedence = typing.List[ConnectionIdentifierPrecedenceEnum]
"""
Order of precedence for attribute types. If the property is not specified, the default precedence of attributes will be used.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .user_attribute_profile_user_attribute_additional_properties import (
    UserAttributeProfileUserAttributeAdditionalProperties,
)

UserAttributeProfileUserAttributes = typing.Dict[str, UserAttributeProfileUserAttributeAdditionalProperties]
"""
User attributes configuration map. Keys are attribute names, values are the mapping configuration for each attribute.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .flow_action_salesforce_create_lead_params_payload import FlowActionSalesforceCreateLeadParamsPayload


class FlowActionSalesforceCreateLeadParams(UniversalBaseModel):
    connection_id: str
    first_name: typing.Optional[str] = None
    last_name: str
    company: str
    email: typing.Optional[str] = None
    phone: typing.Optional[str] = None
    payload: typing.Optional[FlowActionSalesforceCreateLeadParamsPayload] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

ConnectionDomainOkta = str
"""
Domain of the Okta organization (e.g., dev-123456.okta.com). Should be just the domain of the okta server with no scheme or trailing backslash. Discovery runs only when connection.options.oidc_metadata is empty and a domain is provided
"""



# This file was auto-generated by Fern from our API Definition.

ConnectionOpPolicyUri = str
"""
URL that the OpenID Provider provides to the person registering the Client to read about the OPs requirements on how the Relying Party can use the data provided by the OP. The registration process SHOULD display this URL to the person registering the Client if it is given.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

LogDetails = typing.Dict[str, typing.Any]
"""
Additional useful details about this event (structure is dependent upon event type).
"""



# This file was auto-generated by Fern from our API Definition.

ConnectionImportMode = bool
"""
Enables lazy migration mode for importing users from an external database. When a user authenticates, their credentials are validated against the legacy store, then the user is created in Auth0 for future logins.
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_https_url_with_http_fallback_255 import ConnectionHttpsUrlWithHttpFallback255

ConnectionAccessTokenUrloAuth1 = ConnectionHttpsUrlWithHttpFallback255
"""
The URL of the OAuth 1.0a access-token endpoint. This endpoint is used to exchange the temporary request token obtained from the request-token endpoint for an access token during the OAuth 1.0a authentication flow.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_my_organization_configuration_allowed_strategies_enum import (
    ClientMyOrganizationConfigurationAllowedStrategiesEnum,
)
from .client_my_organization_deletion_behavior_enum import ClientMyOrganizationDeletionBehaviorEnum


class ClientMyOrganizationPostConfiguration(UniversalBaseModel):
    """
    Configuration related to the My Organization Configuration for the client.
    """

    connection_profile_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The connection profile ID that this client should validate against.
    """

    user_attribute_profile_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The user attribute profile ID that this client should validate against.
    """

    allowed_strategies: typing.List[ClientMyOrganizationConfigurationAllowedStrategiesEnum] = pydantic.Field()
    """
    The allowed connection strategies for the My Organization Configuration.
    """

    connection_deletion_behavior: ClientMyOrganizationDeletionBehaviorEnum
    invitation_landing_client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The client ID this client uses while creating invitations through My Organization API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_branding_phone_templates_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "branding.phone.templates.list_.0"
    client = get_client(test_id)
    client.branding.phone.templates.list(
        disabled=True,
    )
    verify_request_count(test_id, "GET", "/branding/phone/templates", {"disabled": "true"}, 1)


def test_branding_phone_templates_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "branding.phone.templates.create.0"
    client = get_client(test_id)
    client.branding.phone.templates.create()
    verify_request_count(test_id, "POST", "/branding/phone/templates", None, 1)


def test_branding_phone_templates_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "branding.phone.templates.get.0"
    client = get_client(test_id)
    client.branding.phone.templates.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/branding/phone/templates/id", None, 1)


def test_branding_phone_templates_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "branding.phone.templates.delete.0"
    client = get_client(test_id)
    client.branding.phone.templates.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/branding/phone/templates/id", None, 1)


def test_branding_phone_templates_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "branding.phone.templates.update.0"
    client = get_client(test_id)
    client.branding.phone.templates.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/branding/phone/templates/id", None, 1)


def test_branding_phone_templates_reset() -> None:
    """Test reset endpoint with WireMock"""
    test_id = "branding.phone.templates.reset.0"
    client = get_client(test_id)
    client.branding.phone.templates.reset(
        id="id",
        request={"key": "value"},
    )
    verify_request_count(test_id, "PATCH", "/branding/phone/templates/id/reset", None, 1)


def test_branding_phone_templates_test() -> None:
    """Test test endpoint with WireMock"""
    test_id = "branding.phone.templates.test.0"
    client = get_client(test_id)
    client.branding.phone.templates.test(
        id="id",
        to="to",
    )
    verify_request_count(test_id, "POST", "/branding/phone/templates/id/try", None, 1)
